package	jp.co.sds_corp.ratemixer.fx.filter;

import java.math.BigDecimal;

import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* 為替レートを範囲指定でフィルタリングします。
XMLは以下の形式です。
<PRE>
&lt;filter class="jp.co.sds_corp.ratemixer.fx.filter.RangeFilter"&gt;
    &lt;target ccy1="USD" ccy2="JPY" term="SP"/&gt;
    &lt;target ccy1="EUR" ccy2="JPY" exclude="true"/&gt;
    &lt;max&gt;150&lt;/max&gt;
    &lt;min&gt;80&lt;/min&gt;
&lt;/filter&gt;
</PRE>
*/
public class RangeFilter extends FxFilter
{
	private static final long serialVersionUID = 1L;
	BigDecimal	max;
	BigDecimal	min;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		max = new BigDecimal( config.get( "max" ) );
		min = new BigDecimal( config.get( "min" ) );
	}

	/**
	* レートのフィルタリングを行なう
	* @param	ev		レート変更イベント
	*/
	public boolean isValid( ReceiveEvent ev ) throws Exception {
		FxRate rate = (FxRate )ev.getData();
		if ( rate == null || rate.getBid() == null || rate.getAsk() == null ) {
			return false;
		}
		if ( rate.getBid().compareTo( max ) > 0 ||
			rate.getAsk().compareTo( max ) > 0 ||
			rate.getBid().compareTo( min ) < 0 ||
			rate.getAsk().compareTo( min ) < 0 )
		{
			return false;
		}
		return true;
	}

	/**
	* 最大値(max)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMax( BigDecimal var ) {
		this.max = var ;
	}
	/**
	* 最大値(max)を取得します。
	* @return	maxの内容
	*/
	public BigDecimal getMax() {
		return max;
	}

	/**
	* 最小値(min)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMin( BigDecimal var ) {
		this.min = var ;
	}
	/**
	* 最小値(min)を取得します。
	* @return	minの内容
	*/
	public BigDecimal getMin() {
		return min;
	}

}
