package	jp.co.sds_corp.ratemixer.fx.source;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.math.BigDecimal;
import java.util.Iterator;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* RateFeed取得元からレートを取得します。
*/
public class RateFeed2 extends HttpSource
{
	private static final long serialVersionUID = 1L;

	/**
	* 指定された初期情報でRateFeed2を初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
	}

	/**
	* 取得したHttpResponseを解析してレート情報を取得します。
	* @param	res		HTTP Response文字列
	*/
	public void analyze( String res ) throws Exception {
System.out.println( "res=[" + res + "]" );
		int		nPos = 0;
		Iterator<String> itr = getRateMap().keySet().iterator();
		while ( itr.hasNext() ) {
			String  key = (String )itr.next();
			FxRate rate = (FxRate )getRateMap().get( key );
			FxRate tmp;
			try {
				tmp = (FxRate )parseRate( rate, res, nPos );
			} catch ( Exception ex ) {
				nPos++;
				continue;
			}

			BigDecimal bBid = tmp.getBid();
			BigDecimal bAsk = tmp.getAsk();
			if ( rate.getTerm().equals( "SP" )) {
				if ( bBid.equals( new BigDecimal( "0" ))) {
					nPos++;
					continue;
				}
				BigDecimal bDiv = new BigDecimal( Math.pow( 10.0, bBid.scale() ));
				BigDecimal bTmp;
				bTmp = new BigDecimal( 2 );
				bTmp = bTmp.divide( bDiv, bBid.scale(), BigDecimal.ROUND_DOWN );
				bBid = bBid.subtract( bTmp );
				bAsk = bAsk.add( bTmp );
			}
			tmp.setBid( bBid );
			tmp.setAsk( bAsk );
			if ( !rate.isInit() ) {
			} else {
				fireStatusChange( new StatusChangeEvent( this,
										StatusChangeEvent.STAT_INIT,
										StatusChangeEvent.STAT_RUN,
										"start" ));
				rate.setInit( false );
			}
			getRateMap().put( key, tmp );
			nPos++;
			if ( !rate.equals( tmp ) ) {
				bChg = true;
				fireReceive( tmp );
			}
		}
	}

	private String[] rics = {
		"SP", "ON", "TN", //"TOM", "TDY",
		"1W", "2W", "3W",
		"1M", "2M", "3M", "6M", "9M",
		"1Y", "2Y", "3Y"
	};
	private String[] terms = {
		"SP", "ON", "TN",
		"1W", "2W", "3W",
		"1M", "2M", "3M", "6M", "9M",
		"1Y", "2Y", "3Y"
	};
	private String getRic( FxRate rate ) {
		for ( int i = 0; i < terms.length; i++ ) {
			if ( rate.getTerm().equals( terms[i] )) {
				String ret = rics[i];
				if ( ret.length() == 2 ) {
					ret = ret + " ";
				}
				return ret;
			}
		}
		return "   ";
	}

	/**
	* Http Requestを送信します。
	* @param	os		送信先Stream
	*/
	public void sendRequest( OutputStream os ) throws Exception {
		String req;
		Writer writer = new OutputStreamWriter( os );
		StringBuffer sb = new StringBuffer();
		Iterator<String> itr = getRateMap().keySet().iterator();
		while ( itr.hasNext() ) {
			String  key = (String )itr.next();
			FxRate rate = (FxRate )getRateMap().get( key );
			CcyPair pair = rate.getCcyPair();
			sb.append( pair.getCcy1().getCode() + pair.getCcy2().getCode()
						+ getRic( rate ) );
		}
		req = sb.toString();
/*
System.out.print( ">" );
BufferedReader br = new BufferedReader( new InputStreamReader( System.in ));
req = br.readLine();
*/

System.out.println( "req=[" + req + "]" );
		writer.write( req );
		writer.flush();
		writer.close();
	}

	/**
	* 取得した文字列を解析して、レート情報を返します。
	* @param	rate	レート情報
	* @param	res		取得した文字列
	* @param	nPos	レート位置
	* @return	取得したレートを格納したレート情報
	*/
	protected FxRate parseRate( FxRate rate, String res, int nPos )
		throws Exception
	{
		String[] arr = CmnString.getStringToken( res, "*\t" );

		String bid = arr[nPos*2].trim();
		String ask = arr[nPos*2+1].trim();
		BigDecimal bBid = new BigDecimal( bid );
		BigDecimal bAsk = new BigDecimal( ask );
System.out.println( "[" + bBid + "/" + bAsk + "]" );

		FxRate ret = new FxRate( (FxRate )rate );
		ret.setBid( bBid );
		ret.setAsk( bAsk );
		return ret;
	}
}
